@@ -89,6 +89,7 @@ _gdk_win32_windowing_init (void)
   _gdk_app_hmodule = GetModuleHandle (NULL);
   _gdk_display_hdc = CreateDC ("DISPLAY", NULL, NULL, NULL);
   _gdk_input_locale = GetKeyboardLayout (0);
+  _gdk_win32_keymap_set_active_layout (GDK_WIN32_KEYMAP (_gdk_win32_display_get_keymap (_gdk_display)), _gdk_input_locale);
   _gdk_input_locale_is_ime = ImmIsIME (_gdk_input_locale);
   GetLocaleInfo (MAKELCID (LOWORD (_gdk_input_locale), SORT_DEFAULT),
 		 LOCALE_IDEFAULTANSICODEPAGE,
@@ -97,8 +98,6 @@ _gdk_win32_windowing_init (void)
   GDK_NOTE (EVENTS, g_print ("input_locale:%p, codepage:%d\n",
 			     _gdk_input_locale, _gdk_input_codepage));
 
-  CoInitialize (NULL);
-
   _gdk_selection = gdk_atom_intern_static_string ("GDK_SELECTION");
   _wm_transient_for = gdk_atom_intern_static_string ("WM_TRANSIENT_FOR");
   _targets = gdk_atom_intern_static_string ("TARGETS");
@@ -130,17 +129,17 @@ _gdk_win32_windowing_init (void)
   _cf_url = RegisterClipboardFormat ("UniformResourceLocatorW");
   _cf_html_format = RegisterClipboardFormat ("HTML Format");
   _cf_text_html = RegisterClipboardFormat ("text/html");
-  _is_win8_or_later = g_win32_check_windows_version (6, 2, 0, G_WIN32_OS_ANY);
 
   _gdk_win32_selection_init ();
 }
 
 void
 _gdk_win32_api_failed (const gchar *where,
-		      const gchar *api)
+                       const gchar *api)
 {
-  gchar *msg = g_win32_error_message (GetLastError ());
-  g_warning ("%s: %s failed: %s", where, api, msg);
+  DWORD error_code = GetLastError ();
+  gchar *msg = g_win32_error_message (error_code);
+  g_warning ("%s: %s failed with code %lu: %s", where, api, error_code, msg);
   g_free (msg);
 }
 
@@ -832,6 +831,7 @@ _gdk_win32_message_to_string (UINT msg)
       CASE (WT_PACKET);
       CASE (WT_CSRCHANGE);
       CASE (WT_PROXIMITY);
+      CASE (WM_DPICHANGED);
 #undef CASE
     default:
       if (msg >= WM_HANDHELDFIRST && msg <= WM_HANDHELDLAST)
